@@ -28,6 +28,7 @@
 #include "screen.h"
 #include "client.h"
 #include "widget.h"
+#include "titlebar.h"
 
 extern AwesomeConf globalconf;
 
@@ -154,7 +155,7 @@ ewmh_update_net_client_list(int phys_screen)
     wins = p_new(Window, n);
 
     for(n = 0, c = globalconf.clients; c; c = c->next, n++)
-        if(get_phys_screen(c->screen) == phys_screen)
+        if(c->phys_screen == phys_screen)
             wins[n] = c->win;
 
     XChangeProperty(globalconf.display, RootWindow(globalconf.display, phys_screen),
@@ -239,23 +240,22 @@ ewmh_process_state_atom(Client *c, Atom state, int set)
         {
             c->skiptb = False;
             c->skip = False;
-            c->border = c->oldborder;
         }
         else if(set == _NET_WM_STATE_ADD)
         {
             c->skiptb = True;
             c->skip = True;
-            c->oldborder = c->border;
-            c->border = 0;
         }
     }
     else if(state == net_wm_state_fullscreen)
     {
-        Area geometry = c->geometry;
+        area_t geometry = c->geometry;
         if(set == _NET_WM_STATE_REMOVE)
         {
             /* restore geometry */
             geometry = c->m_geometry;
+            /* restore borders and titlebar */
+            titlebar_position_set(&c->titlebar, c->titlebar.dposition);
             c->border = c->oldborder;
             c->ismax = False;
             client_setfloating(c, c->wasfloating);
@@ -266,6 +266,8 @@ ewmh_process_state_atom(Client *c, Atom state, int set)
             /* save geometry */
             c->m_geometry = c->geometry;
             c->wasfloating = c->isfloating;
+            /* disable titlebar and borders */
+            titlebar_position_set(&c->titlebar, Off);
             c->oldborder = c->border;
             c->border = 0;
             c->ismax = True;
@@ -290,6 +292,7 @@ ewmh_process_window_type_atom(Client *c, Atom state)
         c->border = 0;
         c->skip = True;
         c->isfixed = True;
+        titlebar_position_set(&c->titlebar, Off);
         client_setfloating(c, True);
     }
     else if (state == net_wm_window_type_dialog)
@@ -356,10 +359,11 @@ ewmh_check_client_hints(Client *c)
 NetWMIcon *
 ewmh_get_window_icon(Window w)
 {
+    double alpha;
     NetWMIcon *icon;
     Atom type;
     int format, size, i;
-    unsigned long items, rest, *data, pixel;
+    unsigned long items, rest, *data;
     unsigned char *imgdata, *wdata;
 
     if(XGetWindowProperty(globalconf.display, w,
@@ -392,11 +396,11 @@ ewmh_get_window_icon(Window w)
     icon->image = p_new(unsigned char, size * 4);
     for(imgdata = icon->image, i = 2; i < size + 2; i++, imgdata += 4)
     {
-        pixel = data[i];
-        imgdata[3] = (pixel >> 24) & 0xff; /* A */
-        imgdata[0] = (pixel >> 16) & 0xff; /* R */
-        imgdata[1] = (pixel >>  8) & 0xff; /* G */
-        imgdata[2] = pixel & 0xff;         /* B */
+        imgdata[3] = (data[i] >> 24) & 0xff;           /* A */
+        alpha = imgdata[3] / 255.0;
+        imgdata[2] = ((data[i] >> 16) & 0xff) * alpha; /* R */
+        imgdata[1] = ((data[i] >>  8) & 0xff) * alpha; /* G */
+        imgdata[0] = (data[i]         & 0xff) * alpha; /* B */
     }
 
     XFree(wdata);
